@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,7 +13,7 @@
  * 3. Neither the name of the project nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -35,10 +35,13 @@
  */
 
 #include "ruby/config.h"
+#ifdef RUBY_EXTCONF_H
+#include RUBY_EXTCONF_H
+#endif
 #include <stdio.h>
 #include <sys/types.h>
 #ifndef _WIN32
-#if defined(__BEOS__)
+#if defined(__BEOS__) && !defined(__HAIKU__) && !defined(BONE)
 # include <net/socket.h>
 #else
 # include <sys/socket.h>
@@ -57,6 +60,7 @@
 #endif
 #ifdef _WIN32
 #include <winsock2.h>
+#include <ws2tcpip.h>
 #define snprintf _snprintf
 #endif
 
@@ -67,6 +71,10 @@
 #include <socks.h>
 #endif
 
+#ifndef HAVE_TYPE_SOCKLEN_T
+typedef int socklen_t;
+#endif
+
 #include "addrinfo.h"
 #include "sockport.h"
 
@@ -110,29 +118,10 @@ static struct afd {
 #define ENI_FAMILY	5
 #define ENI_SALEN	6
 
-#ifndef HAVE_INET_NTOP
-static const char *
-inet_ntop(int af, const void *addr, char *numaddr, size_t numaddr_len)
-{
-#ifdef HAVE_INET_NTOA
-	struct in_addr in;
-	memcpy(&in.s_addr, addr, sizeof(in.s_addr));
-	snprintf(numaddr, numaddr_len, "%s", inet_ntoa(in));
-#else
-	unsigned long x = ntohl(*(unsigned long*)addr);
-	snprintf(numaddr, numaddr_len, "%d.%d.%d.%d",
-		 (int) (x>>24) & 0xff, (int) (x>>16) & 0xff,
-		 (int) (x>> 8) & 0xff, (int) (x>> 0) & 0xff);
-#endif
-	return numaddr;
-}
-#endif
-
 int
-getnameinfo(const struct sockaddr *sa, size_t salen, char *host, size_t hostlen, char *serv, size_t servlen, int flags)
+getnameinfo(const struct sockaddr *sa, socklen_t salen, char *host, socklen_t hostlen, char *serv, socklen_t servlen, int flags)
 {
 	struct afd *afd;
-	struct servent *sp;
 	struct hostent *hp;
 	u_short port;
 	int family, len, i;
@@ -148,9 +137,9 @@ getnameinfo(const struct sockaddr *sa, size_t salen, char *host, size_t hostlen,
 	if (sa == NULL)
 		return ENI_NOSOCKET;
 
-	len = SA_LEN(sa);
-	if (len != salen) return ENI_SALEN;
-	
+	if (!VALIDATE_SOCKLEN(sa, salen)) return ENI_SALEN;
+        len = salen;
+
 	family = sa->sa_family;
 	for (i = 0; afdl[i].a_af; i++)
 		if (afdl[i].a_af == family) {
@@ -158,10 +147,10 @@ getnameinfo(const struct sockaddr *sa, size_t salen, char *host, size_t hostlen,
 			goto found;
 		}
 	return ENI_FAMILY;
-	
+
  found:
 	if (len != afd->a_socklen) return ENI_SALEN;
-	
+
 	port = ((struct sockinet *)sa)->si_port; /* network byte order */
 	addr = (char *)sa + afd->a_off;
 
@@ -174,7 +163,7 @@ getnameinfo(const struct sockaddr *sa, size_t salen, char *host, size_t hostlen,
 		strcpy(serv, numserv);
 	} else {
 #if defined(HAVE_GETSERVBYPORT)
-		sp = getservbyport(port, (flags & NI_DGRAM) ? "udp" : "tcp");
+		struct servent *sp = getservbyport(port, (flags & NI_DGRAM) ? "udp" : "tcp");
 		if (sp) {
 			if (strlen(sp->s_name) + 1 > servlen)
 				return ENI_MEMORY;
@@ -193,7 +182,7 @@ getnameinfo(const struct sockaddr *sa, size_t salen, char *host, size_t hostlen,
 			flags |= NI_NUMERICHOST;
 		v4a >>= IN_CLASSA_NSHIFT;
 		if (v4a == 0)
-			flags |= NI_NUMERICHOST;			
+			flags |= NI_NUMERICHOST;
 		break;
 #ifdef INET6
 	case AF_INET6:
